@@ -170,10 +170,10 @@ const ButtonTag = new Lang.Class({
     },
 
     _init: function(params) {
-        this.parent(params);
-
         this._hover = false;
         this._pressed = false;
+
+        this.parent(params);
     },
 
     get hover() {
@@ -1031,7 +1031,7 @@ const ChatView = new Lang.Class({
     _shouldShowStatus: function(nick) {
         let nickTag = this._lookupTag('nick' + nick);
 
-        if (!nickTag)
+        if (!nickTag || !nickTag._lastActivity)
             return false;
 
         let time = GLib.get_monotonic_time();
@@ -1300,7 +1300,7 @@ const ChatView = new Lang.Class({
 
         // mask identify passwords in private chats
         if (this._room.type == Tp.HandleType.CONTACT) {
-            let [isIdentify, username, password] =
+            let [isIdentify, command, username, password] =
                 Polari.util_match_identify_message(text);
 
             if (isIdentify)
