@@ -99,7 +99,8 @@ const MainWindow = new Lang.Class({
                        'userListPopover',
                        'roomListRevealer',
                        'overlay',
-                       'roomStack'],
+                       'roomStack',
+                       'closeConfirmationDialog'],
     Properties: {
         subtitle: GObject.ParamSpec.string('subtitle',
                                            'subtitle',
@@ -133,8 +134,6 @@ const MainWindow = new Lang.Class({
 
         this.parent(params);
 
-        this._addApplicationStyle();
-
         this._settings = new Gio.Settings({ schema_id: 'org.gnome.Polari' });
         this._gtkSettings = Gtk.Settings.get_default();
 
@@ -193,9 +192,19 @@ const MainWindow = new Lang.Class({
                                   Lang.bind(this, this._updateDecorations));
         this._updateDecorations();
 
+        this._closeConfirmationDialog.transient_for = this;
+        this._closeConfirmationDialog.connect('response', (w, r) => {
+            if (r == Gtk.ResponseType.DELETE_EVENT)
+                return;
+
+            this._settings.set_boolean('run-in-background', r == Gtk.ResponseType.ACCEPT);
+            this.destroy();
+        });
+
         this.connect('window-state-event', Lang.bind(this, this._onWindowStateEvent));
         this.connect('size-allocate', Lang.bind(this, this._onSizeAllocate));
         this.connect('destroy', Lang.bind(this, this._onDestroy));
+        this.connect('delete-event', Lang.bind(this, this._onDeleteEvent));
         this.connect('notify::active-room', () => {
             this._updateUserListLabel();
         });
@@ -246,6 +255,32 @@ const MainWindow = new Lang.Class({
             this._settings.reset('last-selected-channel');
     },
 
+    _touchFile: function(file) {
+        try {
+            file.get_parent().make_directory_with_parents(null);
+        } catch(e if e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.EXISTS)) {
+            // not an error, carry on
+        }
+
+        let stream = file.create(0, null);
+        stream.close(null);
+    },
+
+    _onDeleteEvent: function() {
+        let f = Gio.File.new_for_path(GLib.get_user_cache_dir() +
+                                      '/polari/close-confirmation-shown');
+        try {
+            this._touchFile(f);
+        } catch(e) {
+            if (e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.EXISTS))
+                return Gdk.EVENT_PROPAGATE; // the dialog has been shown
+            log('Failed to mark confirmation dialog as shown: ' + e.message);
+        }
+
+        this._closeConfirmationDialog.show();
+        return Gdk.EVENT_STOP;
+    },
+
     _onAccountsChanged: function(am) {
         let hasAccounts = this._accountsMonitor.enabledAccounts.length > 0;
         this._roomListRevealer.reveal_child = hasAccounts;
@@ -314,22 +349,6 @@ const MainWindow = new Lang.Class({
             });
     },
 
-    _addApplicationStyle: function() {
-        let provider = new Gtk.CssProvider();
-        let uri = 'resource:///org/gnome/Polari/css/application.css';
-        let file = Gio.File.new_for_uri(uri);
-        try {
-            provider.load_from_file(Gio.File.new_for_uri(uri));
-        } catch(e) {
-            logError(e, "Failed to add application style");
-        }
-        Gtk.StyleContext.add_provider_for_screen(
-            this.get_screen(),
-            provider,
-            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
-        );
-    },
-
     _onRoomsLoaded: function(mgr) {
         if (this.active_room)
             return;
