@@ -9,6 +9,7 @@ const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const RoomManager = imports.roomManager;
 const Signals = imports.signals;
+const Utils = imports.utils;
 
 const LIST_CHUNK_SIZE = 100;
 
@@ -107,24 +108,30 @@ const RoomListColumn = {
     SENSITIVE: 3,
 };
 
+function _strBaseEqual(str1, str2) {
+    return str1.localeCompare(str2, {}, { sensitivity: 'base'}) == 0;
+};
+
 const ServerRoomList = new Lang.Class({
     Name: 'ServerRoomList',
-    Extends: Gtk.ScrolledWindow,
+    Extends: Gtk.Box,
+    Template: 'resource:///org/gnome/Polari/ui/server-room-list.ui',
+    InternalChildren: ['filterEntry',
+                       'list',
+                       'spinner',
+                       'store',
+                       'toggleRenderer'],
     Properties: { 'can-join': GObject.ParamSpec.boolean('can-join',
                                                         'can-join',
                                                         'can-join',
                                                         GObject.ParamFlags.READABLE,
-                                                        false),
-                  'loading': GObject.ParamSpec.boolean('loading',
-                                                       'loading',
-                                                       'loading',
-                                                       GObject.ParamFlags.READABLE,
                                                        false)
     },
 
     _init: function(params) {
         this._account = null;
         this._pendingInfos = [];
+        this._filterTerms = [];
 
         this.parent(params);
 
@@ -132,49 +139,54 @@ const ServerRoomList = new Lang.Class({
             this.setAccount(null);
         });
 
-        let store = new Gtk.ListStore();
-        store.set_column_types([GObject.TYPE_BOOLEAN,
-                                GObject.TYPE_STRING,
-                                GObject.TYPE_STRING,
-                                GObject.TYPE_BOOLEAN]);
+        this._list.model.set_visible_func((model, iter) => {
+            let name = model.get_value(iter, RoomListColumn.NAME);
+            if (!name)
+                return false;
+
+            if (this._isCustomRoomItem(iter))
+                return true;
+
+            return this._filterTerms.every((term) => name.indexOf(term) != -1);
+        });
+
+        [, this._customRoomItem] = this._store.get_iter_first();
+        this._list.model.refilter();
+
+        this._filterEntry.connect('changed', () => {
+            this._updateCustomRoomName();
+            this._updateSelection();
+        });
+        this._filterEntry.connect('search-changed', () => {
+            if (!Utils.updateTerms(this._filterTerms, this._filterEntry.text))
+                return;
+
+            this._list.model.refilter();
+            this._updateSelection();
+        });
+        this._filterEntry.connect('stop-search', () => {
+            if (this._filterEntry.get_text_length() > 0)
+                this._filterEntry.set_text('');
+            else if (this.get_toplevel() instanceof Gtk.Dialog)
+                this.get_toplevel().response(Gtk.ResponseType.CANCEL);
+        });
+        this._filterEntry.connect('activate', () => {
+            if (this._filterEntry.text.trim().length == 0)
+                return;
+
+            let [selected, model, iter] = this._list.get_selection().get_selected();
+            if (selected)
+                this._toggleChecked(this._list.model.get_path(iter));
+        });
 
-        this._list = new Gtk.TreeView({ model: store,
-                                        activate_on_single_click: true,
-                                        enable_grid_lines: Gtk.TreeViewGridLines.HORIZONTAL,
-                                        headers_visible: false,
-                                        visible: true });
-        this._list.get_style_context().add_class('polari-server-room-list');
         this._list.connect('row-activated', (view, path, column) => {
             this._toggleChecked(path);
         });
-        this.add(this._list);
 
-        let renderer;
-
-        let column = new Gtk.TreeViewColumn();
-        this._list.append_column(column);
-
-        renderer = new Gtk.CellRendererToggle();
-        renderer.connect('toggled', (cell, pathStr) => {
-            this._toggleChecked(Gtk.TreePath.new_from_string(pathString));
+        this._toggleRenderer.connect('toggled', (cell, pathStr) => {
+            this._toggleChecked(Gtk.TreePath.new_from_string(pathStr));
         });
 
-        column.pack_start(renderer, false);
-        column.add_attribute(renderer, 'active', RoomListColumn.CHECKED);
-        column.add_attribute(renderer, 'sensitive', RoomListColumn.SENSITIVE);
-
-        renderer = new Gtk.CellRendererText({ ellipsize: Pango.EllipsizeMode.END });
-
-        column.pack_start(renderer, true);
-        column.add_attribute(renderer, 'text', RoomListColumn.NAME);
-        column.add_attribute(renderer, 'sensitive', RoomListColumn.SENSITIVE);
-
-        renderer = new Gtk.CellRendererText();
-
-        column.pack_start(renderer, false);
-        column.add_attribute(renderer, 'text', RoomListColumn.COUNT);
-        column.add_attribute(renderer, 'sensitive', RoomListColumn.SENSITIVE);
-
         this._manager = getDefault();
         this._manager.connect('loading-changed',
                               Lang.bind(this, this._onLoadingChanged));
@@ -182,7 +194,7 @@ const ServerRoomList = new Lang.Class({
 
     get can_join() {
         let canJoin = false;
-        this._list.model.foreach((model, path, iter) => {
+        this._store.foreach((model, path, iter) => {
             canJoin = model.get_value(iter, RoomListColumn.SENSITIVE) &&
                       model.get_value(iter, RoomListColumn.CHECKED);
             return canJoin;
@@ -190,19 +202,14 @@ const ServerRoomList = new Lang.Class({
         return canJoin;
     },
 
-    get loading() {
-        return this._pendingInfos.length ||
-               (this._account && this._manager.isLoading(this._account));
-    },
-
     get selectedRooms() {
         let rooms = [];
-        let [valid, iter] = this._list.model.get_iter_first();
-        for (; valid; valid = this._list.model.iter_next(iter)) {
-            if (!this._list.model.get_value(iter, RoomListColumn.SENSITIVE) ||
-                !this._list.model.get_value(iter, RoomListColumn.CHECKED))
+        let [valid, iter] = this._store.get_iter_first();
+        for (; valid; valid = this._store.iter_next(iter)) {
+            if (!this._store.get_value(iter, RoomListColumn.SENSITIVE) ||
+                !this._store.get_value(iter, RoomListColumn.CHECKED))
                 continue;
-            rooms.push(this._list.model.get_value(iter, RoomListColumn.NAME));
+            rooms.push(this._store.get_value(iter, RoomListColumn.NAME));
         }
         return rooms;
     },
@@ -213,20 +220,75 @@ const ServerRoomList = new Lang.Class({
 
         this._account = account;
         this._pendingInfos = [];
-        this._list.model.clear();
+        this._clearList();
+        this._filterEntry.set_text('');
         this._onLoadingChanged(this._manager, account);
     },
 
+    focusEntry: function() {
+        this._filterEntry.grab_focus();
+    },
+
+    _isCustomRoomItem: function(iter) {
+        let path = this._store.get_path(iter);
+        let customPath = this._store.get_path(this._customRoomItem);
+        return path.compare(customPath) == 0;
+    },
+
+    _updateCustomRoomName: function() {
+        let newName = this._filterEntry.text.trim();
+        if (newName.search(/\s/) != -1)
+            newName = '';
+
+        if (newName) {
+            let exactMatch = false;
+            this._store.foreach((model, path, iter) => {
+                if (this._isCustomRoomItem(iter))
+                    return false;
+
+                let name = model.get_value(iter, RoomListColumn.NAME);
+                return exactMatch = _strBaseEqual(newName, name);
+            });
+
+            if (exactMatch)
+                newName = '';
+        }
+
+        this._store.set_value(this._customRoomItem, RoomListColumn.NAME, newName);
+    },
+
+    _updateSelection: function() {
+        if (this._filterEntry.text.trim().length == 0)
+            return;
+
+        let model = this._list.model;
+        let [valid, iter] = model.get_iter_first();
+        if (!valid)
+            return;
+
+        this._list.get_selection().select_iter(iter);
+        this._list.scroll_to_cell(model.get_path(iter), null, true, 0.0, 0.0);
+    },
+
+    _clearList: function() {
+        let [valid, iter] = this._store.get_iter_first();
+        if (this._isCustomRoomItem(iter))
+            return;
+        this._store.move_before(this._customRoomItem, iter);
+        while (this._store.remove(iter))
+            ;
+    },
+
     _onLoadingChanged: function(mgr, account) {
         if (account != this._account)
             return;
 
-        this.notify('loading');
+        this._checkSpinner();
 
         if (this.loading)
             return;
 
-        this._list.model.clear();
+        this._clearList();
 
         if (this._idleId)
             Mainloop.source_remove(this._idleId);
@@ -244,45 +306,59 @@ const ServerRoomList = new Lang.Class({
         });
         this._pendingInfos = roomInfos;
 
-        this.notify('loading');
+        this._checkSpinner();
 
         let roomManager = RoomManager.getDefault();
 
         this._idleId = Mainloop.idle_add(() => {
+            let customName = this._store.get_value(this._customRoomItem,
+                                                   RoomListColumn.NAME);
             this._pendingInfos.splice(0, LIST_CHUNK_SIZE).forEach(roomInfo => {
-                let store = this._list.model;
+                let store = this._store;
 
                 let name = roomInfo.get_name();
                 if (name[0] == '#')
                     name = name.substr(1, name.length);
 
-                let room = roomManager.lookupRoomByName(roomInfo.get_name());
+                if (_strBaseEqual(name, customName))
+                    this._store.set_value(this._customRoomItem,
+                                          RoomListColumn.NAME, customName = '');
+
+                let room = roomManager.lookupRoomByName(roomInfo.get_name(), this._account);
                 let sensitive = room == null;
                 let checked = !sensitive;
                 let count = '%d'.format(roomInfo.get_members_count(null));
 
-                store.insert_with_valuesv(-1,
-                                          [RoomListColumn.CHECKED,
-                                           RoomListColumn.NAME,
-                                           RoomListColumn.COUNT,
-                                           RoomListColumn.SENSITIVE],
-                                          [checked, name, count, sensitive]);
+                let iter = store.insert_with_valuesv(-1,
+                                                     [RoomListColumn.CHECKED,
+                                                      RoomListColumn.NAME,
+                                                      RoomListColumn.COUNT,
+                                                      RoomListColumn.SENSITIVE],
+                                                     [checked, name, count, sensitive]);
+                store.move_before(iter, this._customRoomItem);
             });
             if (this._pendingInfos.length)
                 return GLib.SOURCE_CONTINUE;
 
             this._idleId = 0;
-            this.notify('loading');
+            this._checkSpinner();
             return GLib.SOURCE_REMOVE;
         });
     },
 
+    _checkSpinner: function() {
+        let loading = this._pendingInfos.length ||
+                      (this._account && this._manager.isLoading(this._account));
+        this._spinner.active = loading;
+    },
+
     _toggleChecked: function(path) {
-        let [valid, iter] = this._list.model.get_iter(path);
-        if (!this._list.model.get_value(iter, RoomListColumn.SENSITIVE))
+        let childPath = this._list.model.convert_path_to_child_path(path);
+        let [valid, iter] = this._store.get_iter(childPath);
+        if (!this._store.get_value(iter, RoomListColumn.SENSITIVE))
             return;
-        let checked = this._list.model.get_value(iter, RoomListColumn.CHECKED);
-        this._list.model.set_value(iter, RoomListColumn.CHECKED, !checked);
+        let checked = this._store.get_value(iter, RoomListColumn.CHECKED);
+        this._store.set_value(iter, RoomListColumn.CHECKED, !checked);
 
         this.notify('can-join');
     }
