@@ -54,11 +54,19 @@ const _balancedParens = '\\((?:[^\\s()<>]+|(?:\\(?:[^\\s()<>]+\\)))*\\)';
 const _leadingJunk = '[\\s`(\\[{\'\\"<\u00AB\u201C\u2018]';
 const _notTrailingJunk = '[^\\s`!()\\[\\]{};:\'\\".,<>?\u00AB\u00BB\u201C\u201D\u2018\u2019]';
 
+// schemes that only use a colon cannot be matched generically without producing
+// a lot of false positives, so whitelist some useful ones and hope nobody complains :-)
+const _schemeWhitelist = ['geo', 'mailto', 'man', 'info', 'ghelp', 'help'];
+
 const _urlRegexp = new RegExp(
     '(^|' + _leadingJunk + ')' +
     '(' +
         '(?:' +
-            '(?:[a-z]+):' +                       // scheme:
+            '(?:[a-z]+)://' +                     // scheme://
+            '|' +
+            '(?:' +
+                _schemeWhitelist.join('|') +      // scheme:
+            '):' +
             '|' +
             'www\\d{0,3}[.]' +                    // www.
             '|' +
@@ -78,6 +86,14 @@ const _urlRegexp = new RegExp(
 
 const _channelRegexp = new RegExp('(^| )#([\\w\\+\\.-]+)','g');
 
+let _inFlatpakSandbox = undefined;
+
+function isFlatpakSandbox() {
+    if (_inFlatpakSandbox === undefined)
+        _inFlatpakSandbox = GLib.file_test('/.flatpak-info', GLib.FileTest.EXISTS);
+    return _inFlatpakSandbox;
+}
+
 function getTpEventTime() {
     let time = Gtk.get_current_event_time ();
     if (time == 0)
@@ -172,6 +188,20 @@ function openURL(url, timestamp) {
     }
 }
 
+function updateTerms(terms, str) {
+    let normalized = str.trim().toLowerCase().replace(/\s+/g, ' ');
+    let newTerms = normalized ? normalized.split(' ') : [];
+
+    let changed = newTerms.length != terms.length;
+    for (let i = 0; i < terms.length && !changed; i++)
+        changed = terms[i] != newTerms[i];
+
+    if (changed)
+        terms.splice.apply(terms, [0, terms.length, ...newTerms]);
+
+    return changed;
+}
+
 function gpaste(text, title, callback) {
     if (title.length > MAX_PASTE_TITLE_LENGTH)
         title = title.substr(0, MAX_PASTE_TITLE_LENGTH - 1) + '…';
